@@ -1,8 +1,8 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 import os
-import pathlib
 import shutil
+from pathlib import Path
 
 from apppath.utilities import SYSTEM, ensure_existence, get_win_folder
 
@@ -112,116 +112,177 @@ class AppPath(object):
         )
 
     @property
-    def user_data(self) -> pathlib.Path:
+    def user_data(self) -> Path:
         """
         User data path
 
         :return:
         :rtype:"""
-        path = self.user_data_path(
-            self._app_name,
-            self._app_author,
-            version=self._app_version,
-            roaming=self._roaming,
+
+        return ensure_existence(
+            self._user_data_path(
+                self._app_name,
+                self._app_author,
+                version=self._app_version,
+                roaming=self._roaming,
+            ),
+            enabled=self._ensure_existence,
         )
-        ensure_existence(path, enabled=self._ensure_existence)
-        return path
 
     @property
-    def site_data(self) -> pathlib.Path:
+    def site_data(self) -> Path:
         """
         Site data path
 
         :return:
         :rtype:"""
-        path = self.site_data_path(
-            self._app_name,
-            self._app_author,
-            version=self._app_version,
-            multi_path=self._multi_path,
+        return ensure_existence(
+            self._site_data_path(
+                self._app_name,
+                self._app_author,
+                version=self._app_version,
+                multi_path=self._multi_path,
+            ),
+            enabled=self._ensure_existence,
         )
-        ensure_existence(path, enabled=self._ensure_existence)
-        return path
 
     @property
-    def user_config(self) -> pathlib.Path:
+    def user_config(self) -> Path:
         """
         User config path
 
         :return:
         :rtype:"""
-        path = self.user_config_path(
-            self._app_name,
-            self._app_author,
-            version=self._app_version,
-            roaming=self._roaming,
+        return ensure_existence(
+            self._user_config_path(
+                self._app_name,
+                self._app_author,
+                version=self._app_version,
+                roaming=self._roaming,
+            ),
+            enabled=self._ensure_existence,
         )
-        ensure_existence(path, enabled=self._ensure_existence)
-        return path
 
     @property
-    def site_config(self) -> pathlib.Path:
+    def site_config(self) -> Path:
         """
         Site config path
 
         :return:
         :rtype:"""
-        site_config = self.site_config_path(
-            self._app_name,
-            self._app_author,
-            version=self._app_version,
-            multi_path=self._multi_path,
+        return ensure_existence(
+            self._site_config_path(
+                self._app_name,
+                self._app_author,
+                version=self._app_version,
+                multi_path=self._multi_path,
+            ),
+            enabled=self._ensure_existence,
         )
-        ensure_existence(site_config, enabled=self._ensure_existence)
-        return site_config
 
     @property
-    def user_cache(self) -> pathlib.Path:
+    def root_cache(self) -> Path:
+        if SYSTEM == "win32" or SYSTEM == "win32":
+            raise SystemError("Invalid system")
+        return ensure_existence(Path("/var/cache") / self._app_name, enabled=self._ensure_existence)
+
+    @property
+    def root_config(self) -> Path:
+        if SYSTEM == "win32" or SYSTEM == "win32":
+            raise SystemError("Invalid system")
+        return ensure_existence(Path("/etc") / self._app_name, enabled=self._ensure_existence)
+
+    @property
+    def root_log(self) -> Path:
+        if SYSTEM == "win32" or SYSTEM == "win32":
+            raise SystemError("Invalid system")
+        return ensure_existence(Path("/var/log") / self._app_name, enabled=self._ensure_existence)
+
+    @property
+    def root_state(self) -> Path:
+        if SYSTEM == "win32" or SYSTEM == "win32":
+            raise SystemError("Invalid system")
+        return ensure_existence(Path("/var/lib") / self._app_name, enabled=self._ensure_existence)
+
+    @property
+    def root_run(self) -> Path:
+        if SYSTEM == "win32" or SYSTEM == "win32":
+            raise SystemError("Invalid system")
+        return ensure_existence(Path("/run") / self._app_name, enabled=self._ensure_existence)
+
+    @property
+    def root_tmp(self) -> Path:
+        if SYSTEM == "win32" or SYSTEM == "win32":
+            raise SystemError("Invalid system")
+        return ensure_existence(Path("/tmp") / self._app_name, enabled=self._ensure_existence)
+
+    @property
+    def root_long_tmp(self) -> Path:
+        if SYSTEM == "win32" or SYSTEM == "win32":
+            raise SystemError("Invalid system")
+        return ensure_existence(Path("/var/tmp") / self._app_name, enabled=self._ensure_existence)
+
+    @property
+    def site_cache(self) -> Path:
+        """
+        TODO: Nonsense, is same as user_cache
+        Returns:
+
+        """
+        return self.user_cache
+
+    @property
+    def site_log(self) -> Path:
+        """
+        TODO: Nonsense, is same as user_log
+        Returns:
+
+        """
+        return self.user_log
+
+    @property
+    def user_cache(self) -> Path:
         """
         User cache path
 
         :return:
         :rtype:"""
-        path = self.user_cache_path(
-            self._app_name, self._app_author, version=self._app_version
+        return ensure_existence(
+            self._user_cache_path(self._app_name, self._app_author, version=self._app_version),
+            enabled=self._ensure_existence,
         )
-        ensure_existence(path, enabled=self._ensure_existence)
-        return path
 
     @property
-    def user_state(self) -> pathlib.Path:
+    def user_state(self) -> Path:
         """
         User state path
 
         :return:
         :rtype:"""
-        path = self.user_state_path(
-            self._app_name, self._app_author, version=self._app_version
+        return ensure_existence(
+            self._user_state_path(self._app_name, self._app_author, version=self._app_version),
+            enabled=self._ensure_existence,
         )
-        ensure_existence(path, enabled=self._ensure_existence)
-        return path
 
     @property
-    def user_log(self) -> pathlib.Path:
+    def user_log(self) -> Path:
         """
         User log path
 
         :return:
         :rtype:"""
-
-        path = self.user_log_path(
-            self._app_name, self._app_author, version=self._app_version
+        return ensure_existence(
+            self._user_log_path(self._app_name, self._app_author, version=self._app_version),
+            enabled=self._ensure_existence,
         )
-        ensure_existence(path, enabled=self._ensure_existence)
-        return path
 
     @staticmethod
-    def user_data_path(
+    def _user_data_path(
         app_name: str = None,
         app_author: str = None,
         version: str = None,
         roaming: bool = False,
-    ) -> pathlib.Path:
+    ) -> Path:
         r"""Return full path to the user-specific data dir for this application.
 
         "app_name" is the name of application.
@@ -265,20 +326,18 @@ class AppPath(object):
             if app_author is None:
                 app_author = app_name
             const = roaming and "CSIDL_APPDATA" or "CSIDL_LOCAL_APPDATA"
-            path_ = pathlib.Path(os.path.normpath(get_win_folder(const)))
+            path_ = Path(os.path.normpath(get_win_folder(const)))
             if app_name:
                 if app_author is not False:
                     path_ = path_ / app_author / app_name
                 else:
                     path_ /= app_name
         elif SYSTEM == "darwin":
-            path_ = pathlib.Path.home() / "Library" / "Application Support"
+            path_ = Path.home() / "Library" / "Application Support"
             if app_name:
                 path_ /= app_name
         else:
-            path_ = pathlib.Path(
-                os.getenv("XDG_DATA_HOME", pathlib.Path.home() / ".local" / "share")
-            )
+            path_ = Path(os.getenv("XDG_DATA_HOME", Path.home() / ".local" / "share"))
             if app_name:
                 path_ /= app_name
         if app_name and version:
@@ -286,12 +345,12 @@ class AppPath(object):
         return path_
 
     @staticmethod
-    def site_data_path(
+    def _site_data_path(
         app_name: str = None,
         app_author: str = None,
         version: str = None,
         multi_path: bool = False,
-    ) -> pathlib.Path:
+    ) -> Path:
         r"""Return full path to the user-shared data dir for this application.
 
         "app_name" is the name of application.
@@ -324,37 +383,30 @@ class AppPath(object):
         if SYSTEM == "win32":
             if app_author is None:
                 app_author = app_name
-            path = pathlib.Path(
-                os.path.normpath(get_win_folder("CSIDL_COMMON_APPDATA"))
-            )
+            path = Path(os.path.normpath(get_win_folder("CSIDL_COMMON_APPDATA")))
             if app_name:
                 if app_author is not False:
                     path = path / app_author / app_name
                 else:
                     path /= app_name
         elif SYSTEM == "darwin":
-            path = pathlib.Path.home() / "Library" / "Application Support"
+            path = Path.home() / "Library" / "Application Support"
             if app_name:
                 path /= app_name
         else:
             # XDG default for $XDG_DATA_DIRS
             # only first, if multipath is False
-            path = os.getenv(
-                "XDG_DATA_DIRS", os.pathsep.join(["/usr/local/share", "/usr/share"])
-            )
-            path_list = [
-                os.path.expanduser(x.rstrip(os.sep)) for x in path.split(os.pathsep)
-            ]
+            path = os.getenv("XDG_DATA_DIRS", os.pathsep.join(["/usr/local/share", "/usr/share"]))
+            path_list = [os.path.expanduser(x.rstrip(os.sep)) for x in path.split(os.pathsep)]
             if app_name:
                 if version:
-                    app_name = os.path.join(app_name, version)
-                path_list = [os.sep.join([x, app_name]) for x in path_list]
-
-            path_list = [pathlib.Path(a) for a in path_list]
+                    app_name = Path(app_name) / version
+                path_list = [Path(x) / app_name for x in path_list]
 
             if multi_path:
-                path = os.pathsep.join(path_list)
+                path = os.pathsep.join([str(a) for a in path_list])
             else:
+                path_list = [Path(a) for a in path_list]
                 path = path_list[0]
             return path
 
@@ -363,12 +415,12 @@ class AppPath(object):
         return path
 
     @staticmethod
-    def user_config_path(
+    def _user_config_path(
         app_name: str = None,
         app_author: str = None,
         version: str = None,
         roaming: bool = False,
-    ) -> pathlib.Path:
+    ) -> Path:
         r"""Return full path to the user-specific config dir for this application.
 
         "app_name" is the name of application.
@@ -397,15 +449,13 @@ class AppPath(object):
         For Unix, we follow the XDG spec and support $XDG_CONFIG_HOME.
         That means, by default "~/.config/<AppName>"."""
         if SYSTEM == "win32":
-            path = AppPath.user_data_path(app_name, app_author, None, roaming)
+            path = AppPath._user_data_path(app_name, app_author, None, roaming)
         elif SYSTEM == "darwin":
-            path = pathlib.Path.home() / "Library" / "Preferences"
+            path = Path.home() / "Library" / "Preferences"
             if app_name:
                 path /= app_name
         else:
-            path = pathlib.Path(
-                os.getenv("XDG_CONFIG_HOME", pathlib.Path.home() / ".config")
-            )
+            path = Path(os.getenv("XDG_CONFIG_HOME", Path.home() / ".config"))
             if app_name:
                 path /= app_name
         if app_name and version:
@@ -413,12 +463,12 @@ class AppPath(object):
         return path
 
     @staticmethod
-    def site_config_path(
+    def _site_config_path(
         app_name: str = None,
         app_author: str = None,
         version: str = None,
         multi_path: bool = False,
-    ) -> pathlib.Path:
+    ) -> Path:
         r"""Return full path to the user-shared data dir for this application.
 
         "app_name" is the name of application.
@@ -448,40 +498,37 @@ class AppPath(object):
 
         WARNING: Do not use this on Windows. See the Vista-Fail note above for why."""
         if SYSTEM == "win32":
-            path = AppPath.site_data_path(app_name, app_author)
+            path = AppPath._site_data_path(app_name, app_author)
             if app_name and version:
                 path /= version
         elif SYSTEM == "darwin":
-            path = pathlib.Path.home() / "Library" / "Preferences"
+            path = Path.home() / "Library" / "Preferences"
             if app_name:
                 path /= app_name
         else:
             # XDG default for $XDG_CONFIG_DIRS
             # only first, if multi_path is False
             path = os.getenv("XDG_CONFIG_DIRS", "/etc/xdg")
-            path_list = [
-                os.path.expanduser(x.rstrip(os.sep)) for x in path.split(os.pathsep)
-            ]
+            path_list = [os.path.expanduser(x.rstrip(os.sep)) for x in path.split(os.pathsep)]
             if app_name:
                 if version:
-                    app_name = os.path.join(app_name, version)
-                path_list = [os.sep.join([x, app_name]) for x in path_list]
-
-            path_list = [pathlib.Path(a) for a in path_list]
+                    app_name = Path(app_name) / version
+                path_list = [Path(x) / app_name for x in path_list]
 
             if multi_path:
-                path = os.pathsep.join(path_list)
+                path = os.pathsep.join([str(a) for a in path_list])
             else:
+                # path_list = [Path(a) for a in path_list]
                 path = path_list[0]
         return path
 
     @staticmethod
-    def user_cache_path(
+    def _user_cache_path(
         app_name: str = None,
         app_author: str = None,
         version: str = None,
         opinionated: bool = True,
-    ) -> pathlib.Path:
+    ) -> Path:
         r"""Return full path to the user-specific cache dir for this application.
 
         "appname" is the name of application.
@@ -517,7 +564,7 @@ class AppPath(object):
         if SYSTEM == "win32":
             if app_author is None:
                 app_author = app_name
-            path = pathlib.Path(os.path.normpath(get_win_folder("CSIDL_LOCAL_APPDATA")))
+            path = Path(os.path.normpath(get_win_folder("CSIDL_LOCAL_APPDATA")))
             if app_name:
                 if app_author is not False:
                     path = path / app_author / app_name
@@ -526,13 +573,11 @@ class AppPath(object):
                 if opinionated:
                     path /= "Cache"
         elif SYSTEM == "darwin":
-            path = pathlib.Path.home() / "Library" / "Caches"
+            path = Path.home() / "Library" / "Caches"
             if app_name:
                 path /= app_name
         else:
-            path = pathlib.Path(
-                os.getenv("XDG_CACHE_HOME", pathlib.Path.home() / ".cache")
-            )
+            path = Path(os.getenv("XDG_CACHE_HOME", Path.home() / ".cache"))
             if app_name:
                 path /= app_name
         if app_name and version:
@@ -540,12 +585,12 @@ class AppPath(object):
         return path
 
     @staticmethod
-    def user_state_path(
+    def _user_state_path(
         app_name: str = None,
         app_author: str = None,
         version: str = None,
         roaming: bool = False,
-    ) -> pathlib.Path:
+    ) -> Path:
         r"""Return full path to the user-specific state dir for this application.
 
         "app_name" is the name of application.
@@ -576,11 +621,9 @@ class AppPath(object):
 
         That means, by default "~/.local/state/<AppName>"."""
         if SYSTEM in ["win32", "darwin"]:
-            path = AppPath.user_data_path(app_name, app_author, None, roaming)
+            path = AppPath._user_data_path(app_name, app_author, None, roaming)
         else:
-            path = pathlib.Path(
-                os.getenv("XDG_STATE_HOME", pathlib.Path.home() / ".local" / "state")
-            )
+            path = Path(os.getenv("XDG_STATE_HOME", Path.home() / ".local" / "state"))
             if app_name:
                 path /= app_name
         if app_name and version:
@@ -588,12 +631,12 @@ class AppPath(object):
         return path
 
     @staticmethod
-    def user_log_path(
+    def _user_log_path(
         app_name: str = None,
         app_author: str = None,
         version: str = None,
         opinionated: bool = True,
-    ) -> pathlib.Path:
+    ) -> Path:
         r"""Return full path to the user-specific log dir for this application.
 
         "app_name" is the name of application.
@@ -626,14 +669,14 @@ class AppPath(object):
         value for Windows and appends "log" to the user cache dir for Unix.
         This can be disabled with the `opinionated=False` option."""
         if SYSTEM == "darwin":
-            path = pathlib.Path.home() / "Library" / "Logs" / app_name
+            path = Path.home() / "Library" / "Logs" / app_name
         elif SYSTEM == "win32":
-            path = AppPath.user_data_path(app_name, app_author, version)
+            path = AppPath._user_data_path(app_name, app_author, version)
             version = False
             if opinionated:
                 path /= "Logs"
         else:
-            path = AppPath.user_cache_path(app_name, app_author, version)
+            path = AppPath._user_cache_path(app_name, app_author, version)
             version = False
             if opinionated:
                 path /= "log"
@@ -657,38 +700,47 @@ class AppPath(object):
             self.clean_site_data()
 
     def clean_user_log(self) -> None:
+        """"""
         if self.user_log.exists():
             shutil.rmtree(self.user_log)
 
     def clean_user_config(self) -> None:
+        """"""
         if self.user_config.exists():
             shutil.rmtree(self.user_config)
 
     def clean_user_data(self) -> None:
+        """"""
         if self.user_data.exists():
             shutil.rmtree(self.user_data)
 
     def clean_user_cache(self) -> None:
+        """"""
         if self.user_cache.exists():
             shutil.rmtree(self.user_cache)
 
     def clean_site_config(self) -> None:
+        """"""
         if self.site_config.exists():
             shutil.rmtree(self.site_config)
 
     def clean_site_data(self) -> None:
+        """"""
         if self.site_data.exists():
             shutil.rmtree(self.site_data)
 
     @property
     def app_name(self):
+        """"""
         return self._app_name
 
     @property
     def app_author(self):
+        """"""
         return self._app_author
 
     def app_version(self):
+        """"""
         return self._app_version
 
 
@@ -710,26 +762,24 @@ if __name__ == "__main__":
         )
 
         print("-- app dirs (with optional 'version')")
-        dirs = AppPath(
-            _app_name, _app_author, app_version="1.0", ensure_existence_on_access=False
-        )
+        dirs = AppPath(_app_name, _app_author, app_version="1.0", ensure_existence_on_access=False)
         for prop in props:
-            print("%s: %s" % (prop, getattr(dirs, prop)))
+            print(f"{prop}: {getattr(dirs, prop)}")
 
         print("\n-- app dirs (without optional 'version')")
         dirs = AppPath(_app_name, _app_author, ensure_existence_on_access=False)
         for prop in props:
-            print("%s: %s" % (prop, getattr(dirs, prop)))
+            print(f"{prop}: {getattr(dirs, prop)}")
 
         print("\n-- app dirs (without optional '_app_author')")
         dirs = AppPath(_app_name, ensure_existence_on_access=False)
         for prop in props:
-            print("%s: %s" % (prop, getattr(dirs, prop)))
+            print(f"{prop}: {getattr(dirs, prop)}")
 
         print("\n-- app dirs (with disabled '_app_author')")
         dirs = AppPath(_app_name, ensure_existence_on_access=False)
         for prop in props:
-            print("%s: %s" % (prop, getattr(dirs, prop)))
+            print(f"{prop}: {getattr(dirs, prop)}")
 
         ex = None
         try:
